// Copyright 2024 Esri
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//   https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import SwiftUI
import ArcGIS

private class Model: ObservableObject {
    // MARK: Graphics

    // The graphics overlay to add graphics to.
    let graphicsOverlay: GraphicsOverlay = {
        let graphicsOverlay = GraphicsOverlay()
        graphicsOverlay.sceneProperties.surfacePlacement = .absolute
        graphicsOverlay.sceneProperties.altitudeOffset = 5
        return graphicsOverlay
    }()

    // The symbol to indicate the observer.
    private let observerSymbol: Symbol = {
        let symbol = SimpleMarkerSymbol(
            style: .circle,
            color: .black,
            size: 20
        )
        symbol.outline = SimpleLineSymbol(
            style: .solid,
            color: .white,
            width: 2
        )
        return symbol
    }()

    // The symbol to indicate the target.
    private var targetSymbol: Symbol = {
        let symbol = SimpleMarkerSymbol(
            style: .circle,
            color: .white,
            size: 20
        )
        symbol.outline = SimpleLineSymbol(
            style: .solid,
            color: .black,
            width: 2
        )
        return symbol
    }()



    // MARK: Analysis
    // The analysis overlay to be added to the scene.
    let analysisOverlay = AnalysisOverlay()

    // The location of the observer.
    private var observerLocation: Point?

    // Creates the line of sight with the scene point.
    func createLineOfSight(scenePoint: Point) {

        if let observerLocation {
            // Creates and set initial line of sight analysis with the tapped scene point.
            let lineOfSight = LocationLineOfSight(observerLocation: observerLocation, targetLocation: scenePoint)
            // Adds the line of sight to the analysis overlay.
            analysisOverlay.addAnalysis(lineOfSight)
            // Creates a target graphic and adds it to the graphics overlay.
            let targetGraphic = Graphic(geometry: scenePoint, symbol: targetSymbol)
            graphicsOverlay.addGraphic(targetGraphic)
        }

        else {
            // Updates the observer location.
            observerLocation = scenePoint
            // Adds the observer graphic to the graphics overlay.
            let observerGraphic = Graphic(geometry: scenePoint, symbol: observerSymbol)
            graphicsOverlay.addGraphic(observerGraphic)
        }

    }

    // Resets the observer location and clears all graphics and analyses.
    func clearAnalyses() {
        analysisOverlay.removeAllAnalyses()
        graphicsOverlay.removeAllGraphics()
        observerLocation = nil
    }

}

struct ContentView: View {

    // An ObservableObject containing the scene, graphics overlay, and analysis overlay.
    @StateObject private var model = Model()

    // The Yosemite Valley hotspots scene.
    @State private var scene: ArcGIS.Scene = {

        let portalItem = PortalItem(
            portal: .arcGISOnline(connection: .anonymous),
            id: Item.ID("7558ee942b2547019f66885c44d4f0b1")!
        )

        return Scene(item: portalItem)
    }()

    var body: some View {

        SceneView(scene: scene, graphicsOverlays: [model.graphicsOverlay], analysisOverlays: [model.analysisOverlay])

            .onSingleTapGesture { _, scenePoint in
                // Places an observer point upon single touch.
                guard let scenePoint else { return }
                model.createLineOfSight(scenePoint: scenePoint)
            }

            .toolbar {
                ToolbarItemGroup(placement: .bottomBar) {
                    Button("Clear") {
                        // Resets the line of sight.
                        model.clearAnalyses()
                    }
                }
            }

    }

}


